@@ -21,6 +21,7 @@
 
 #include <string.h>
 #include "window.h"
+#include "class-group.h"
 #include "xutils.h"
 #include "private.h"
 #include "wnck-enum-types.h"
@@ -49,6 +50,7 @@ struct _WnckWindowPrivate
   Window xwindow;
   WnckScreen *screen;
   WnckApplication *app;
+  WnckClassGroup *class_group;
   Window group_leader;
   Window transient_for;
   char *name;
@@ -71,6 +73,11 @@ struct _WnckWindowPrivate
   int y;
   int width;
   int height;
+
+  char *startup_id;
+
+  char *res_class;
+  char *res_name;
   
   /* true if transient_for points to root window,
    * not another app window
@@ -108,7 +115,9 @@ struct _WnckWindowPrivate
   guint need_emit_icon_changed : 1;
   guint need_update_actions : 1;
   guint need_update_wintype : 1;
-  guint need_update_transient_for : 1;  
+  guint need_update_transient_for : 1;
+  guint need_update_startup_id : 1;
+  guint need_update_wmclass : 1;
 };
 
 enum {
@@ -144,6 +153,8 @@ static void update_workspace (WnckWindow *window);
 static void update_actions   (WnckWindow *window);
 static void update_wintype   (WnckWindow *window);
 static void update_transient_for (WnckWindow *window);
+static void update_startup_id (WnckWindow *window);
+static void update_wmclass    (WnckWindow *window);
 static void unqueue_update   (WnckWindow *window);
 static void queue_update     (WnckWindow *window);
 static void force_update_now (WnckWindow *window);
@@ -280,7 +291,10 @@ wnck_window_finalize (GObject *object)
   g_free (window->priv->name);
   g_free (window->priv->icon_name);
   g_free (window->priv->session_id);
-
+  g_free (window->priv->startup_id);
+  g_free (window->priv->res_class);
+  g_free (window->priv->res_name);
+  
   if (window->priv->app)
     g_object_unref (G_OBJECT (window->priv->app));
 
@@ -375,6 +389,8 @@ _wnck_window_create (Window      xwindow,
   window->priv->need_update_actions = TRUE;
   window->priv->need_update_wintype = TRUE;
   window->priv->need_update_transient_for = TRUE;
+  window->priv->need_update_startup_id = TRUE;
+  window->priv->need_update_wmclass = TRUE;
   force_update_now (window);
 
   return window;
@@ -461,6 +477,14 @@ wnck_window_get_xid (WnckWindow *window)
   return window->priv->xwindow;
 }
 
+WnckClassGroup *
+wnck_window_get_class_group (WnckWindow *window)
+{
+  g_return_val_if_fail (WNCK_IS_WINDOW (window), NULL);
+
+  return window->priv->class_group;
+}
+
 /**
  * wnck_window_get_session_id:
  * @window: a #WnckWindow
@@ -571,6 +595,45 @@ wnck_window_is_maximized_vertically   (WnckWindow *window)
   return window->priv->is_maximized_vert;
 }
 
+const char*
+_wnck_window_get_startup_id (WnckWindow *window)
+{
+  g_return_val_if_fail (WNCK_IS_WINDOW (window), NULL);
+  
+  if (window->priv->startup_id == NULL &&
+      window->priv->group_leader != None)
+    {
+      WnckApplication *app;
+
+      /* Fall back to group leader property */
+      
+      app = wnck_application_get (window->priv->group_leader);
+      
+      if (app != NULL)
+        return wnck_application_get_startup_id (app);
+      else
+        return NULL;
+    }
+
+  return window->priv->startup_id;
+}
+
+const char*
+_wnck_window_get_resource_class (WnckWindow *window)
+{
+  g_return_val_if_fail (WNCK_IS_WINDOW (window), NULL);
+
+  return window->priv->res_class;
+}
+
+const char*
+_wnck_window_get_resource_name (WnckWindow *window)
+{
+  g_return_val_if_fail (WNCK_IS_WINDOW (window), NULL);
+
+  return window->priv->res_name;
+}
+
 /**
  * wnck_window_is_maximized:
  * @window: a #WnckWindow
@@ -1245,6 +1308,20 @@ _wnck_window_set_application (WnckWindow      *window,
   window->priv->app = app;
 }
 
+void
+_wnck_window_set_class_group (WnckWindow     *window,
+			      WnckClassGroup *class_group)
+{
+  g_return_if_fail (WNCK_IS_WINDOW (window));
+  g_return_if_fail (class_group == NULL || WNCK_IS_CLASS_GROUP (class_group));
+
+  if (class_group)
+    g_object_ref (G_OBJECT (class_group));
+  if (window->priv->class_group)
+    g_object_unref (G_OBJECT (window->priv->class_group));
+  window->priv->class_group = class_group;
+}
+
 void
 _wnck_window_process_property_notify (WnckWindow *window,
                                       XEvent     *xevent)
@@ -1306,6 +1383,17 @@ _wnck_window_process_property_notify (WnckWindow *window,
       window->priv->need_update_wintype = TRUE;
       queue_update (window);
     }
+  else if (xevent->xproperty.atom ==
+           _wnck_atom_get ("_NET_STARTUP_ID"))
+    {
+      window->priv->need_update_startup_id = TRUE;
+      queue_update (window);
+    }
+  else if (xevent->xproperty.atom == XA_WM_CLASS)
+    {
+      window->priv->need_update_wmclass = TRUE;
+      queue_update (window);
+    }
   else if (xevent->xproperty.atom ==
            _wnck_atom_get ("_NET_WM_ICON") ||
            xevent->xproperty.atom ==
@@ -1684,7 +1772,7 @@ update_wintype (WnckWindow *window)
             type = WNCK_WINDOW_MODAL_DIALOG;
           else if (atoms[i] == _wnck_atom_get ("_NET_WM_WINDOW_TYPE_UTILITY"))
             type = WNCK_WINDOW_UTILITY;
-          else if (atoms[i] == _wnck_atom_get ("_NET_WM_WINDOW_TYPE_SPLASHSCREEN"))
+          else if (atoms[i] == _wnck_atom_get ("_NET_WM_WINDOW_TYPE_SPLASH"))
             type = WNCK_WINDOW_SPLASHSCREEN;
           else
             found_type = FALSE;
@@ -1739,6 +1827,36 @@ update_transient_for (WnckWindow *window)
     }
 }
 
+static void
+update_startup_id (WnckWindow *window)
+{
+  if (!window->priv->need_update_startup_id)
+    return;
+
+  window->priv->need_update_startup_id = FALSE;
+
+  g_free (window->priv->startup_id);
+  window->priv->startup_id = 
+    _wnck_get_utf8_property (window->priv->xwindow,
+                             _wnck_atom_get ("_NET_STARTUP_ID"));
+}
+
+static void
+update_wmclass (WnckWindow *window)
+{
+  if (!window->priv->need_update_wmclass)
+    return;
+
+  window->priv->need_update_wmclass = FALSE;
+
+  g_free (window->priv->res_class);
+  g_free (window->priv->res_name);
+
+  _wnck_get_wmclass (window->priv->xwindow,
+                     &window->priv->res_class,
+                     &window->priv->res_name);
+}
+
 static void
 force_update_now (WnckWindow *window)
 {
@@ -1786,6 +1904,8 @@ force_update_now (WnckWindow *window)
   old_state = COMPRESS_STATE (window);
   old_actions = window->priv->actions;
 
+  update_startup_id (window);    /* no side effects */
+  update_wmclass (window);
   update_transient_for (window); /* wintype needs this to be first */
   update_wintype (window);
   update_wm_state (window);
